;------------------------------------------------------------------
; Blue propeller clock
;
; TODO V0.1:
; - S Inititalization
; + S Calibration mode
; + S fix dot counter origin/sync
; + S main loop
; + S characterset in EEPROM
; + S separate fixed positions for date display
; - H finalize board
; - H motor pinout
; - H rotary transformer hardware, PSU
; - H balancing
;
; TODO V0.2:
; - S control with IR remote
; - S set time/date (IR)
; - S display modes
; - H remote controller
;
; FUTURE ENHANCEMENTS:
; - light sensor
; - temp sensor
; - written time
; - animated digit transitions
; - use as 20 char IR text display
; - IR firmware update
; - multiple character sets
;------------------------------------------------------------------
	processor 16F88
	include	<16F88.INC>

;**	__config  B'11110100000001'

	radix	dec
	errorlevel -302,-202	;turn off bank switch warnings
Xtal	equ	20e+6		;Crystal frequency
	include	<PINS.INC>	;pinout
	include	<VARIABLE.INC>	;ram registers
	include	<MACRO.INC>	;macros
;------------------------------------------------------------------
; PROGRAM START ORIGIN
;------------------------------------------------------------------
	org	H'0000'		;start at PCL 0000
	call	init		;initialization
	call	cal		;calibration
	goto	start		;go!
	nop
;------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------
irq	btfss	TMR0IF		;TMR0 interrupt?
	goto	irq_in		;no, it was another source
	bcf	S_OE		;yes, LEDs off without affecting STATUS
	bcf	TMR0IE		;clear interrupt ENABLE flag
	retfie			;fast return from interrupt

irq_in	movwf	w_temp		;save W
	swapf	STATUS,W	;save STATUS
	movwf	s_temp
	movff	FSR,f_temp	;save FSR
	clrf	STATUS		;make sure we're in bank 0
	bsf	IRP		; except indirect adressing

irq_t2	btfss	TMR2IF		;was it TMR2?
	goto	irq_int		;no, must've been INT0
	decfsz	pre,F		;keep time prescaler
	goto	irq_end
	movlf	rlpre,pre	;prescaler overflowed
	call	keep_time	;update time and date
	goto	irq_end		;done

irq_int	bcf	INT0IF		;clear INT0 source flag
	incf	dot,F		;keep dot counter
	bcf	dot,7		;keep it in the 0-127 range
	btfss	DOT_FR		;zero-frame passed?
	goto	irq_syn		;yes, sync dot count
	bcf	SYNC		;no, reset pass bit
	goto	port_bf		;and continue
	
irq_syn	btfsc	SYNC		;did this already?
	goto	port_bf		;yes, then continue
	movff	origin,dot	;sync dot clock
	bsf	SYNC		;set SYNC bit, so this only happens once

port_bf	movf	dot,W		;get dot number
	call	getdata		;get buffer byte
	movwf	a_temp		;put away
	movf	dot,W		;get dot number
	addlw	H'40'		;add half a frame
	call	getdata		;get buffer byte
	movwf	b_temp		;put away
	call	leds		;shift into HC595s and do other stuff
	movff	light,TMR0	;set intensity level
	bcf	TMR0IF		;clear TMR0 interrupt flag (hasn't been done yet)
	bsf	TMR0IE		;enable next interrupt

irq_end	movff	f_temp,FSR	;recall all registers
	swapf	s_temp,W	;recall STATUS register
	movwf	STATUS
	swapf	w_temp,F	;restore W without affecting status bits
	swapf	w_temp,W
	retfie			;return from interrupt
;------------------------------------------------------------------
; MAIN PROGRAM
; user interface (display mode, set time)
; buffer updates
;------------------------------------------------------------------
start	bsf	GIE		;start display scanning
main	btfss	TXTMODE		;display text?
	goto	main_02		;no, numbers
main_01	btfsc	MINUTE		;1 minute passed?
	call	text_buffer	;update text buffer
	goto	main_03
main_02	btfsc	DATE		;date update?
	call	date_buffer	;yes, then update buffer
	btfsc	TIME		;time update?
	call	time_buffer	;update time buffer
main_03	btfss	RCIF		;IR input?
	goto	main		;loop
;------------------------------------------------------------------
; PROCESS IR REMOTE COMMANDS
;------------------------------------------------------------------
ir	movff	RCREG,cmd
	bcf	RCIF		;reset reception flag
	sublw	cr		;received a CR?
	btfsc	Z
	goto	expert		;yes, go to expert mode
	movf	cmd,W
	sublw	c_right		;cursor right?
	btfsc	Z
	incf	mode,F		;next display mode
	movf	cmd,W
	sublw	c_left		;cursor left?
	btfss	Z
	decf	mode,F		;previous display mode
	return
;------------------------------------------------------------------
expert	return
;------------------------------------------------------------------
; INITIALIZATION
; timer0 = LED intensity level
;------------------------------------------------------------------
init	clrf	STATUS		;bank 0
	call	clrmem		;clear memory
	call	shift		;clear leds

	clrf	PORTB
	movlf	B'01001111',T2CON	;TMR2 post=10 pre=16
	movlf	B'11000001',ADCON0	;int RC / AN0

	bsf	RP0			;bank 1
	movlf	B'01110000',INTCON	;TMR0, INT0,
	bsf	TMR2IE			; and T2 generate interrupts
	movlf	B'11101011',TRISA
	movlf	B'00000011',TRISB
	movlf	B'00100000',ADCON1	;use vref+
	movlf	B'xxxxxxxx',OPTION_REG	;TMR0pre=1
	movlf	249,PR2			;set TMR2 period (125 Hz)
	movlf	B'00000011',ANSEL	;select AN0, AN1

	bcf	RP0			;bank 0
	movlf	rlpre,pre	;load prescaler
	bsf	IRP		;all indirect addressing in bank 2+3
	return
;------------------------------------------------------------------
; set time and date
;------------------------------------------------------------------
setup	movlf	month,FSR	;set pointer to first location
setrpt	call	getdigt		;get first digit
	movf	a_temp,W	;make copy in W
	bcf	C		;clear carry
	rlf	a_temp,F	;temp=temp*2	*2
	rlf	a_temp,F	;temp=temp*2	*4
	addwf	a_temp,F	;temp=temp+W	*5
	rlf	a_temp,W	;W=temp*2	*10
	movwf	INDF		;put tens in place
	call	getdigt		;get digit
	movf	a_temp,W
	addwf	INDF,F		;add to tens
	movlf	255,PORTB	;flash LEDs
	call	debnc
	incfw	FSR		;next location
	sublw	seconds+1	;last one?
	btfss	Z
	goto	setrpt		;no

	clrf	PORTB		;leds off
	movf	month,W		;month=0?
	btfss	Z		;yes, then month=1
	decf	month,F		;month=month-1
	movf	date,W		;date=0?
	btfss	Z		;yes, then date=1
	decf	date,F		;date=date-1
	movf	day,W		;day=0?
	andlw	7
	btfss	Z		;yes, then day=1
	addlw	-1		;day=day-1
	movwf	day
	call	days_tm		;set days_this_month
	call	time_buffer	;prepare initial time buffer
	goto	date_buffer	;prepare initial date buffer
;------------------------------------------------------------------
; inclusions
;------------------------------------------------------------------
	include	<routines.inc>
	include	<charset.inc>
;------------------------------------------------------------------
	end
;------------------------------------------------------------------
